!********************************************************************************
!** FICHE F.28.  CONSTANT-NVT MOLECULAR DYNAMICS - EXTENDED SYSTEM METHOD      **
!** This FORTRAN code is intended to illustrate points made in the text.       **
!** To our knowledge it works correctly.  However it is the responsibility of  **
!** the user to test it, if it is to be used in a research application.        **
!   (Modified by Yoshi, Jan 2005)
!    Comment:
!    Equation of motion in REAL system is solved.
!********************************************************************************

!C    *******************************************************************
!C    ** CONSTANT-NVT MOLECULAR DYNAMICS USING AN EXTENDED SYSTEM.     **
!C    **                                                               **
!C    ** REFERENCE:                                                    **
!C    **                                                               **
!C    ** NOSE, MOLEC. PHYS. 52, 255, 1984.                             **
!C    **                                                               **
!C    ** ROUTINES:                                                     **
!C    **                                                               **
!C    ** SUBROUTINE PREDIC                                             **
!C    **    PREDICTS POSITIONS AT TIME T + DT                          **
!C    ** SUBROUTINE CORREC                                             **
!C    **    CORRECTS POSITIONS AT TIME T + DT                          **
!C    **                                                               **
!C    ** PRINCIPAL VARIABLES:                                          **
!C    **                                                               **
!C    ** INTEGER N                             NUMBER OF ATOMS         **
!C    ** REAL    DT                            TIMESTEP                **
!C    ** REAL    RX(N),RY(N),RZ(N)             ATOM POSITIONS          **
!C    ** REAL    VX(N),VY(N),VZ(N)             FIRST DERIVATIVES       **
!C    ** REAL    AX(N),AY(N),AZ(N)             SECOND DERIVATIVES      **
!C    ** REAL    BX(N),BY(N),BZ(N)             THIRD DERIVATIVES       **
!C    ** REAL    FX(N),FY(N),FZ(N)             TOTAL FORCES            **
!C    ** REAL    S,SV,SA,SB,SC,SF              S AND DERIVATIVES       **
!C    ** REAL    WQ                            THERMAL INERTIA         **
!C    **                                                               **
!C    ** USAGE:                                                        **
!C    **                                                               **
!C    ** THE MODIFIED EQUATIONS OF MOTION ARE AS FOLLOWS:              **
!C    ** A = F/WM - SV*V/S                                             **
!C    ** SA = SUM (WM*S/WQ)*V**2  - 3N*KT*S/WQ +SV**2/S                **
!C    ** WHERE R,V,A,.. ARE SUCCESSIVE DERIVATIVES OF POSITION         **
!C    ** AND SV,SA,.... ARE SUCCESSIVE DERIVATIVES OF EXTRA VARIABLE S **
!C    ** F REPRESENTS FORCES, WM PARTICLE MASS, WQ EXTRA VARIABLE MASS **
!C    ** KT IS TEMPERATURE*BOLTZMANN'S CONSTANT                        **
!C    ** AND WE HAVE (3N-3) DEGREES OF FREEDOM IF MOMENTUM FIXED.      **
!C    ** IN A SENSE, S IS A TIME-SCALING FACTOR.                       **
!C    ** WE SOLVE THESE EQUATIONS BY A GEAR 5-VALUE METHOD FOR SECOND  **
!C    ** ORDER DIFFERENTIAL EQUATIONS.                                 **
!C    ** THIS PROGRAM USES UNIT 10 FOR CONFIGURATION INPUT AND OUTPUT  **
!C    *******************************************************************

!C    *******************************************************************
!C    ** STANDARD TAYLOR SERIES PREDICTORS                             **
!C    *******************************************************************

        SUBROUTINE gear_pc_nvt_predic
        use variables
        use constants
        implicit double precision (a-h,o-z)

        INTEGER :: I
        double precision :: C1, C2, C3, C4

!C    *******************************************************************

        C1 = DT
        C2 = C1 * DT / 2.0
        C3 = C2 * DT / 3.0
        C4 = C3 * DT / 4.0

        DO 100 I = 1, N

           RX(I) = RX(I) + C1*VX(I) + C2*AX(I) + C3*BX(I) + C4*CX(I)
           RY(I) = RY(I) + C1*VY(I) + C2*AY(I) + C3*BY(I) + C4*CY(I)
           RZ(I) = RZ(I) + C1*VZ(I) + C2*AZ(I) + C3*BZ(I) + C4*CZ(I)
           VX(I) = VX(I) + C1*AX(I) + C2*BX(I) + C3*CX(I)
           VY(I) = VY(I) + C1*AY(I) + C2*BY(I) + C3*CY(I)
           VZ(I) = VZ(I) + C1*AZ(I) + C2*BZ(I) + C3*CZ(I)
           AX(I) = AX(I) + C1*BX(I) + C2*CX(I)
           AY(I) = AY(I) + C1*BY(I) + C2*CY(I)
           AZ(I) = AZ(I) + C1*BZ(I) + C2*CZ(I)
           BX(I) = BX(I) + C1*CX(I)
           BY(I) = BY(I) + C1*CY(I)
           BZ(I) = BZ(I) + C1*CZ(I)

100     CONTINUE

        S  = S  + C1*SV + C2*SA + C3*SB + C4*SC
        SV = SV + C1*SA + C2*SB + C3*SC
        SA = SA + C1*SB + C2*SC
        SB = SB + C1*SC

        RETURN
        END

!C    *******************************************************************
!C    ** GEAR CORRECTOR ALGORITHM                                      **
!C    **                                                               **
!C    ** WE USE GEAR'S ALTERNATIVE COEFFICIENT GEAR0 WHICH APPLIES IN  **
!C    ** THE CASE THAT FIRST TIME DERIVATIVES APPEAR ON THE RIGHT OF   **
!C    ** THESE SECOND-ORDER DIFFERENTIAL EQUATIONS.                    **
!C    ** FOR TIMESTEP-SCALED VARIABLES THE COEFFICIENTS WOULD BE       **
!C    ** 19/90, 3/4, 1, 1/2, 1/12.                                     **
!C    **                                                               **
!C    ** REFERENCES:                                                   **
!C    **                                                               **
!C    ** GEAR, NUMERICAL INITIAL VALUE PROBLEMS IN ORDINARY            **
!C    ** DIFFERENTIAL EQUATIONS (PRENTICE-HALL, 1971).                 **
!C    ** GEAR, REPORT ANL 7126 (ARGONNE NATIONAL LAB., 1966).          **
!C    *******************************************************************

        SUBROUTINE gear_pc_nvt_correc
        use variables
        use constants
        implicit double precision (a-h,o-z)

        double precision :: GEAR0, GEAR1, GEAR3, GEAR4
        PARAMETER ( GEAR0 = 19.0 / 90.0, GEAR1 = 3.0 /4.0, &
        GEAR3 = 1.0 / 2.0,   GEAR4 = 1.0 /12.0 )

!C    *******************************************************************

        enkin = 0.0; enkineach=0.0
        DO I = 1, N
           enkineach(i) = 0.5d0 * wm(i) * (VX(I) ** 2 + VY(I) ** 2 + VZ(I) ** 2)
           enkin = enkin + enkineach(i)
        ENDDO
        SF = 2.0d0*enkin*s/wq - 3.0d0*dble(N)*bl*temp_set*S/WQ + SV**2/S

        C1 = DT
        C2 = C1 * DT / 2.0
        C3 = C2 * DT / 3.0
        C4 = C3 * DT / 4.0

        CR = GEAR0 * C2
        CV = GEAR1 * C2 / C1
        CB = GEAR3 * C2 / C3
        CC = GEAR4 * C2 / C4

        DO 400 I = 1, N

           AXI = FX(I) / WM(I)  -  SV * VX(I) / S
           AYI = FY(I) / WM(I)  -  SV * VY(I) / S
           AZI = FZ(I) / WM(I)  -  SV * VZ(I) / S
           CORRX = AXI - AX(I)
           CORRY = AYI - AY(I)
           CORRZ = AZI - AZ(I)

           RX(I) = RX(I) + CR * CORRX
           RY(I) = RY(I) + CR * CORRY
           RZ(I) = RZ(I) + CR * CORRZ
           VX(I) = VX(I) + CV * CORRX
           VY(I) = VY(I) + CV * CORRY
           VZ(I) = VZ(I) + CV * CORRZ
           AX(I) = AXI
           AY(I) = AYI
           AZ(I) = AZI
           BX(I) = BX(I) + CB * CORRX
           BY(I) = BY(I) + CB * CORRY
           BZ(I) = BZ(I) + CB * CORRZ
           CX(I) = CX(I) + CC * CORRX
           CY(I) = CY(I) + CC * CORRY
           CZ(I) = CZ(I) + CC * CORRZ

400     CONTINUE

        CORR = SF - SA
        S  = S  + CR * CORR
        SV = SV + CV * CORR
        SA = SF
        SB = SB + CB * CORR
        SC = SC + CC * CORR

        RETURN
        END


